% empirical_kernel_map - Empirical kernel map for polynomial,Gaussian, exponential,
%                        sigmoid, Laplace, multiquadratic and inverse multiquadratic kernels.
%           
%

% INPUTS:
%   X       - Stupci su uzorci, mapiranje se radi za svaki stupac posebno
%   V       - baza u ulaznom prostoru podataka na iju sliku se projicira
%             preslikani skup podataka
%
%   kernel  - Odabir kernela.
%               'polynomial'
%               'gaussian'
%               'exponential'
%               'sigmoid'
%               'laplace'
%               'multiquad'
%               'invmultiquad'
%  correction   0/1 flag: used to correct bias (DC)term introduced by polynomial, exponential and Gaussian kernels
%               0 - no correction; 1 - correction.  
%
%   var1    - Parametar kernela
%               za poly kernel:     stupanj (d = var1)
%               za gaussov kernel:  sigma   (sigma2 = var1)
%   var2    - Parametar kernela
%               za poly kernel:     offset  (c = var2)
%
%
%OUTPUT:
% 
% PSI     Empirical kernel map of X
%
% COPYRIGHT: I. Kopriva, ZLAIR, IRB, Zagreb, Croatia
% DATE:      2012/12/18; 

function PSI = empirical_kernel_map(X,V,kernel,correction,var1,var2)


% INITIALIZATION

if(var1<=0)
    error('Parameter var1 must be positive.');
end


% parameters of kernel
switch(kernel)
    case 'polynomial'
        dp = var1;       % degree
        bp = var2;       % offset
        if(bp<0)
            error('Parameter c (offset) must be nonnegative.');
        end
    
    case 'gaussian'
        sig2 = var1;   
        
    case 'exponential'
        sig2 = var1;
        
    case 'sigmoid'
        sig = var1;
        
    case 'laplace'
        sig2 = var1;
        
    case 'multiquad'
        c = var1;
        
    case 'invmultiquad'
        c = var1;
        
    otherwise
        error('Unknown kernel')
end

TL = size(X,2);
D = size(V,2);

if( strcmp(kernel,'polynomial'))
         PSI = power(V'*X + bp,dp); 
         if correction == 1
            PSI = PSI - ones(D,TL); % correction term to remove ones
         end
    elseif ( strcmp(kernel,'gaussian'))
         g = waitbar(0,'Subspace projection ...');
         for d=1:D
              P = X-repmat(V(:,d),1,TL);   
              P = sum(P.*P);
              PSI(d,:)=exp(-P/2/sig2);
              waitbar(d/D,g)
         end
         close(g)
         % correction term
         if correction == 1
            for d=1:D
                  eps(d,1)= exp(-V(:,d)'*V(:,d)/2/sig2);
            end
            for t= 1:TL
                expxt(t,1) = exp(-X(:,t)'*X(:,t)/2/sig2);
            end
            PSI = PSI - repmat(eps,1,TL)*diag(expxt);
            PSI = max(PSI,0);
         end
    elseif ( strcmp(kernel,'exponential'))
           PSI = exp(V'*X/2/sig2);    
           if correction == 1
                PSI = PSI - ones(D,TL);
           end
    elseif ( strcmp(kernel,'sigmoid'))
           PSI = tanh(V'*X/sig + c);
    elseif ( strcmp(kernel,'laplace'))
           g = waitbar(0,'Subspace projection ...');
           for d=1:D
               P = X-repmat(V(:,d),1,TL);   
               P = sqrt(sum(P.*P));
               PSI(d,:)=exp(-P/2/sig2);
               waitbar(d/D,g)
           end
           close(g)
          elseif ( strcmp(kernel,'multiquad'))
            g = waitbar(0,'Subspace projection ...');
           for d=1:D
               P=X-repmat(V(:,d),1,TL);
               PSI(d,:)=sqrt(sum(P.*P)+c);
               waitbar(d/D,g)
           end
           close(g)
    elseif ( strcmp(kernel,'invmultiquad'))
           g = waitbar(0,'Subspace projection ...');
           for d=1:D
                P = X-repmat(V(:,d),1,TL);     
                PSI(d,:)=1./sqrt(sum(P.*P)+c);
                waitbar(d/D,g)
           end
           close(g)
    end
